using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version  2025
/// 
/// <summary>
/// Piirretään paljon palloja ja tönitään niitä.
/// Lisätty pistelaskuri ja lopetus 10 palloon.
/// </summary>
public class Pallot : PhysicsGame
{
    private IntMeter pisteLaskuri;

    /// <summary>
    /// Luodaan ja lisätään ruudulle annetun kokoinen pallo
    /// annettuun paikkaan.
    /// </summary>
    /// <param name="peli">Peli, johon neliö piirretään</param>
    /// <param name="x">Kappaleen keskipisteen x-koordinaatti</param>
    /// <param name="y">Kappaleen keskipisteen y-koordinaatti</param>
    /// <param name="r">Pallon säde</param>
    /// <param name="vari">Pallon väri</param>
    public static PhysicsObject PiirraPallo(Game peli, double x, double y, double r, Color vari)
    {
        PhysicsObject kappale = new PhysicsObject(2 * r, 2 * r, Shape.Ellipse);
        kappale.Mass = 0.1;
        kappale.Position = new Vector(x, y);
        kappale.Color = vari;
        peli.Add(kappale);
        return kappale;
    }


    /// <summary>
    /// Pelin aloitus, reunat, zoomaus pllaojen luonti ja näppäinten kuuntelu
    /// </summary>
    public override void Begin()
    {
        Image[] kuvat = LoadImages("Baby", "Chell", "Cigur", "Igor", "Infant", "Jango", "Knive", "Leader", "Mum", "Ron", "Ven", "Yar");
        List<Image> kuvatL = new List<Image>(kuvat);
        Image arvottuKuva = RandomGen.SelectOne<Image>(kuvat);

        Camera.ZoomToLevel();
        Level.CreateBorders();
        Level.Background.Color = Color.Black;
        Level.Background.Image = LoadImage("village");
        //  Level.Background.CreateGradient(Color.White, Color.Blue);
        Gravity = new Vector(0, -50);

        for (int i = 0; i < 50; i++)
        {
            PhysicsObject p; // Tämä apumuuttuja tarvitaan Guru-tehtävää varten
            p = PiirraPallo(
                this,
                RandomGen.NextDouble(Level.Left, Level.Right), // Kentän vasen ja oikea reuna
                RandomGen.NextDouble(Level.Bottom, Level.Top), // alareuna, yläreuna
                RandomGen.NextDouble(5, 30),
                RandomGen.NextColor()
                );
            p.Tag = "vihollinen"; // jotta voidaan antaa tunniste törmäyksille
            p.Image = kuvat[RandomGen.NextInt(kuvat.Length)];
        }


        PhysicsObject pallo = PiirraPallo(this, 0, 0, 40, Color.White);
        pallo.Mass = 4;
        AddCollisionHandler(pallo, "vihollinen", PalloTormasi); // Guru tehtävää varten oman pallon ja muiden törmäys
        pallo.Image = LoadImage("King");

        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä näppäinohjeet");
        Keyboard.Listen(Key.Up, ButtonState.Pressed, LyoPalloa, "Lyö palloa ylöspäin", pallo, new Vector(0, 1000));
        Keyboard.Listen(Key.Down, ButtonState.Pressed, LyoPalloa, "Lyö palloa alaspäin", pallo, new Vector(0, -1000));
        Keyboard.Listen(Key.Left, ButtonState.Pressed, LyoPalloa, "Lyö palloa vasemmalle", pallo, new Vector(-1000, 0));
        Keyboard.Listen(Key.Right, ButtonState.Pressed, LyoPalloa, "Lyö palloa oikealle", pallo, new Vector(1000, 0));
        LuoLaskuri();
    }


    /// <summary>
    /// Luo pistelaskurin näyttöön
    /// </summary>
    private void LuoLaskuri()
    {
        pisteLaskuri = new IntMeter(0);
        pisteLaskuri.MaxValue = 10;
        pisteLaskuri.UpperLimit += delegate { PeliPaattyi(pisteLaskuri.MaxValue); };

        Label pisteNaytto = new Label();
        pisteNaytto.X = Screen.Left + 60;
        pisteNaytto.Y = Screen.Top - 10;
        pisteNaytto.TextColor = Color.Brown;
        pisteNaytto.Color = Color.Yellow;
        pisteNaytto.BorderColor = Color.Black;
        pisteNaytto.Width = 100;
        // pisteNaytto.IntFormatstring = "Pisteet: {0,2:D} ";

        pisteNaytto.BindTo(pisteLaskuri);
        Add(pisteNaytto);
    }


    /// <summary>
    /// Annetaan pallolle isku pyydettyyn suuntaan ja pyydetyllä voimalla
    /// </summary>
    /// <param name="pallo">pallo jota lyödään</param>
    /// <param name="suunta">suunta/voimavektori lyöntiä varten</param>
    private void LyoPalloa(PhysicsObject pallo, Vector suunta)
    {
        pallo.Hit(suunta);
    }


    /// <summary>
    /// Suoritetaan kun peli päättyi
    /// </summary>
    /// <param name="arvo">mikä oli laskurin arvo lun tultiin tänne</param>
    private void PeliPaattyi(int arvo)
    {
        MessageDisplay.TextColor = Color.White;
        MessageDisplay.BackgroundColor = Color.Red;
        MessageDisplay.Clear();
        ClearGameObjects();
        MessageDisplay.Add("");
        MessageDisplay.Add("");
        MessageDisplay.Add("");
        MessageDisplay.Add(String.Format("       Hyvä, sait {0} palloa          ", arvo));
        MessageDisplay.Add("");
        MessageDisplay.Add("");
        MessageDisplay.Add("");
    }

    /// <summary>
    /// Guru-tehtävää varten pallojen törmäysten käsittely
    /// </summary>
    /// <param name="pallo">Oma pallo</param>
    /// <param name="kohde">pallo johon törmättiin</param>
    private void PalloTormasi(PhysicsObject pallo, PhysicsObject kohde)
    {
        if (kohde.Y < 0) return;
        Explosion rajahdys = new Explosion(kohde.Width);
        rajahdys.Position = kohde.Position;
        Add(rajahdys);
        kohde.Destroy();
        pisteLaskuri.Value++;
    }
}
