using Jypeli;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version 2025
///
/// <summary>
/// Demo 6, Bonus 5.
/// Game of life, jossa luodaan uusia populaatioita edellisestä kuvasta.
/// Ks esim: http://en.wikipedia.org/wiki/Conway%27s_Game_of_Life
/// </summary>
public class B5_GameOfLife : Game
{

    /// <summary>
    /// Ruutujen määrä y-akselilla
    /// </summary>
    private const int NY = 60;
    private int[,] sukupolvi;
    private int[,] seuraavaSukupolvi;
    private GameObject[,] oliot;
    private Timer timer = new Timer();
    private readonly Color[] varit = { Color.Black, Color.White };

    /// <summary>
    /// Alustetaan peli
    /// </summary>
    public override void Begin()
    {
        Level.Background.Color = Color.Black;

        double dy = Screen.Height / NY; // Lasketaan yhden (elämä-)ruudun korkeus pikseleinä
        int nx = (int)(Screen.Width / dy);  // ja montako mahtuu X-suuntaan
        int ny = NY;

        sukupolvi = new int[ny, nx]; // Luodaan taulukot
        seuraavaSukupolvi = new int[ny, nx];
        oliot = new GameObject[ny, nx];
        // seuraavaSukupolvi = sukupolvi; // jos tämä on mukana, käyttäytyy eri tavalla

        LuoOliot(this, oliot);

        Camera.ZoomToAllObjects();
        IsFullScreen = false;

        timer.Interval = 0.1; // timeri antamaan tapahtuma 0.1 sek välein
        timer.Timeout += LaskeJaPiirraSeuraavaSukupolvi;
        timer.Start();

        ArvoSukupolvi();  // jos halutaan käynnistää automaattisesti

        // Kuuntelijat:
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
        Keyboard.Listen(Key.Enter, ButtonState.Pressed, delegate { timer.Start(); }, "Jatka");
        Keyboard.Listen(Key.Space, ButtonState.Pressed, ArvoSukupolvi, "Arvo sukupolvi");
        Keyboard.Listen(Key.Delete, ButtonState.Pressed, TyhjennaSukupolvi, "Tyhjennä sukupolvi");
        Keyboard.Listen(Key.Add, ButtonState.Pressed, () => { timer.Interval /= 2; }, null);
        Keyboard.Listen(Key.Subtract, ButtonState.Pressed, () => { timer.Interval *= 2; }, null);
        Mouse.IsCursorVisible = true;
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, HiiriPainettu, null);
    }


    /// <summary>
    /// Lisätään peliin ruudukollinen suorakaiteita.
    /// </summary>
    /// <param name="game">peli johon lisätään</param>
    /// <param name="oliot">taulukko johon oliviitteet laitetaan</param>
    private static void LuoOliot(Game game, GameObject[,] oliot)
    {
        int ny = oliot.GetLength(0);
        int nx = oliot.GetLength(1);

        for (int iy = 0; iy < ny; iy++)    // luodaan ruutuoliot
            for (int ix = 0; ix < nx; ix++)
            {
                GameObject obj = new GameObject(1, 1, Shape.Rectangle);
                oliot[iy, ix] = obj; // jemmataan taulukkoon
                obj.X = ix;
                obj.Y = iy;
                obj.Color = Color.Black;
                game.Add(obj);
                // Mouse.ListenOn(obj, MouseButton.Left, ButtonState.Pressed, VaihdaVari, null, iy,ix);
            }
    }


    /// <summary>
    /// Arvotaan uusi sukupolvi
    /// </summary>
    private void ArvoSukupolvi()
    {
        Sopulit.Arvo(sukupolvi, 0, 1);
        timer.Start();
    }


    /// <summary>
    /// Nollataan sukupolvi
    /// </summary>
    private void TyhjennaSukupolvi()
    {
        Sopulit.Tayta(sukupolvi, 0);
        LaskeJaPiirraSeuraavaSukupolvi(); // jotta näyttö muuttuu
    }


    /// <summary>
    /// Käsitellään paikassa iy,ox oleva panallus
    /// </summary>
    /// <param name="paikkaMaailmassa">koordinaatti missä painettiin</param>
    private void Painettu(Vector paikkaMaailmassa)
    {
        int ix = (int)(paikkaMaailmassa.X + 0.5);
        int iy = (int)(paikkaMaailmassa.Y + 0.5);
        if (ix < 0 || iy < 0) return;
        if (ix >= oliot.GetLength(1) || iy >= oliot.GetLength(0)) return;
        VaihdaVari(iy, ix);
    }


    /// <summary>
    /// Käsitellään hiiren painallus
    /// </summary>
    private void HiiriPainettu()
    {
        Painettu(Mouse.PositionOnWorld);
    }

    /// <summary>
    /// Vaihdetaan paikassa iy,ix oleva arvo päinvastaikseksi.
    /// Sammutetaan samalla kello.
    /// </summary>
    /// <param name="iy">y-indeksi paikalle</param>
    /// <param name="ix">x-indeksi paikalle</param>
    private void VaihdaVari(int iy, int ix)
    {
        timer.Stop();
        sukupolvi[iy, ix] ^= 1;
        oliot[iy, ix].Color = varit[sukupolvi[iy, ix]];
    }



    /// <summary>
    /// Lasketaan seuraava vaihe ja vaihdetaan kuvioiden värit vastaavaksi.
    /// </summary>
    private void LaskeJaPiirraSeuraavaSukupolvi()
    {
        Sopulit.SeuraavaSukupolvi(sukupolvi, seuraavaSukupolvi);
        // vaihdetaan taulukot seuraavaa
        // kierrosta varten
        // Tällä tavalla vältetään turhien olioiden tekemistä ylen määrin. 
        int[,] tmp = sukupolvi;
        sukupolvi = seuraavaSukupolvi;
        seuraavaSukupolvi = tmp;

        int ny = sukupolvi.GetLength(0);
        int nx = sukupolvi.GetLength(1);

        for (int iy = 0; iy < ny; iy++)
            for (int ix = 0; ix < nx; ix++)
                oliot[iy, ix].Color = varit[sukupolvi[iy, ix]];
    }
}
