using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version 2021
/// 
/// <summary>
/// Matriisin suurin, pienin ja jonoksi muuttaminen
/// </summary>
public class Matriisit
{

    /// <summary>
    /// Etsitään matriisin suurin alkio
    /// </summary>
    /// <param name="a">tutkittava matriisi</param>
    /// <returns>matriisin suurin alkio</returns>
    /// <example>
    /// <pre name="test">
    ///  double[,] mat1 = {{1,2,3},{2,2,2},{4,2,3}};
    ///  Matriisit.Suurin(mat1) ~~~ 4.0;
    ///  double[,] mat2 = {{9,2,8},{1,2,5},{3,19,-3}};
    ///  Matriisit.Suurin(mat2) ~~~ 19.0;
    ///  double[,] mat3 = new double[0,0];
    ///  Matriisit.Suurin(mat3) ~~~ Double.MinValue;
    /// </pre>
    /// </example>
    public static double Suurin(double[,] a)
    {
        double suurin = Double.MinValue;
        foreach (double luku in a)
            if (luku > suurin) suurin = luku;
        return suurin;
    }


    /// <summary>
    /// Etsitään matriisin pienin alkio
    /// </summary>
    /// <param name="a">tutkittava matriisi</param>
    /// <returns>matriisin pienin alkio</returns>
    /// <example>
    /// <pre name="test">
    ///  double[,] mat1 = {{1,2,3},{2,2,2},{4,2,3}};
    ///  Matriisit.Pienin(mat1) ~~~ 1.0;
    ///  double[,] mat2 = {{9,2,8},{1,2,5},{3,19,-3}};
    ///  Matriisit.Pienin(mat2) ~~~ -3.0;
    ///  double[,] mat3 = new double[0,0];
    ///  Matriisit.Pienin(mat3) ~~~ Double.MaxValue;
    /// </pre>
    /// </example>
    public static double Pienin(double[,] a)
    {
        double pienin = Double.MaxValue;
        for (int iy = 0; iy < a.GetLength(0); iy++)
            for (int ix = 0; ix < a.GetLength(1); ix++)
            {
                double luku = a[iy, ix];
                if (luku < pienin) pienin = luku;
            }

        return pienin;
    }


    /// <summary>
    /// Palauttaa taulukon merkkijonona niin, että alkioiden välissä on erotin.
    /// Alkioden formaatin voi valita itse, samoin mitä tulostetaan rivien väliin
    /// </summary>
    /// <param name="luvut">Taulukko josta tehdään merkkijono</param>
    /// <param name="sarakeErotin">Jono, jolla rivin alkiot erotetaan toisistaan</param>
    /// <param name="sarakeformaatti"></param>
    /// <param name="rivierotin">JOno, jolla rivit erotetaan toisistaan</param>
    /// <param name="riviformaatti">C# tyylinen formaattojono, jolla yksi rivi käsitellään.
    /// Jos formaattijonossa ei ole lainkaan {-merkkiä, käytetään jonoa sellaisenaan rivien
    /// erottomina
    /// </param>
    /// <example>
    /// <pre name="test">
    ///  double[,] luvut = {{1,2,3},{4,5,6},{7,8,9}};
    ///  Matriisit.Jonoksi(luvut) === "1 2 3\n4 5 6\n7 8 9";
    ///  Matriisit.Jonoksi(luvut," ","{0}",",") === "1 2 3,4 5 6,7 8 9";
    ///  Matriisit.Jonoksi(luvut,":","{0:0.0}","|","[ {0} ]") === "[ 1.0:2.0:3.0 ]|[ 4.0:5.0:6.0 ]|[ 7.0:8.0:9.0 ]";
    ///  Matriisit.Jonoksi((double[,])null) === "null";
    ///  Matriisit.Jonoksi(new double[0,0]) === "";
    ///  Matriisit.Jonoksi(new double[1,0]) === "";
    ///  Matriisit.Jonoksi(new double[2,0]) === "\n";
    /// </pre>
    /// </example>
    public static string Jonoksi(double[,] luvut, string sarakeErotin = " ",
                                 string sarakeformaatti = "{0}",
                                 string riviErotin = "\n",
                                 string riviformaatti = "{0}")
    {
        if (luvut == null) return "null";
        StringBuilder tulos = new StringBuilder();
        string formaatti = riviformaatti;
        if (formaatti.IndexOf("{") < 0) formaatti = "{0}" + formaatti;
        string rivivali = "";

        for (int iy = 0; iy < luvut.GetLength(0); iy++)
        {
            string vali = "";
            StringBuilder rivi = new StringBuilder();
            for (int ix = 0; ix < luvut.GetLength(1); ix++)
            {
                rivi.Append(vali);
                rivi.Append(String.Format(sarakeformaatti, luvut[iy, ix]).Replace(",", "."));
                vali = sarakeErotin;
            }
            tulos.Append(rivivali + String.Format(formaatti, rivi));
            rivivali = riviErotin;
        }
        return tulos.ToString();
    }


    /// <summary>
    /// Palauttaa taulukon merkkijonona niin, että alkioiden välissä on erotin.
    /// Alkioden formaatin voi valita itse.
    /// </summary>
    /// <param name="luvut">Taulukko josta tehdään merkkijono</param>
    /// <param name="sarakeErotin">Jono, jolla rivin alkiot erotetaan toisistaan</param>
    /// <param name="sarakeformaatti"></param>
    /// <example>
    /// <pre name="test">
    ///  double[] luvut = {1,2,3};
    ///  Matriisit.Jonoksi(luvut) === "1 2 3";
    ///  Matriisit.Jonoksi(luvut, ":", "{0:0.0}") === "1.0:2.0:3.0";
    ///  Matriisit.Jonoksi((double[]) null) === "null";
    ///  Matriisit.Jonoksi(new double[0]) === "";
    /// </pre>
    /// </example>
    public static string Jonoksi(double[] luvut, string sarakeErotin = " ", string sarakeformaatti = "{0}")
    {
        if (luvut == null) return "null";
        string vali = "";
        StringBuilder rivi = new StringBuilder();
        for (int ix = 0; ix < luvut.Length; ix++)
        {
            rivi.Append(vali);
            rivi.Append(String.Format(sarakeformaatti, luvut[ix]).Replace(",", "."));
            vali = sarakeErotin;
        }
        return rivi.ToString();
    }
}

