﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version 01.10.2011
/// @version 15.10.2011
/// @version 26.2.2013 Muuttujien uudelleennimeämistä (a-j)
/// <summary>
/// Tutkitaan elämäpeliä. 
/// </summary>
/// <remarks>
/// Säännöt ovat oletuksena b3s23 (born 3/stay 2,3), mutta 
/// saaannot-taulukolla voidaan
/// viedä peliin muitakin sääntöjä.
/// </remarks>
public class Sopulit
{
    private static readonly int[,] b3s23 =
                           {//  0 1 2 3 4 5 6 7 8
                                   {0,0,0,1,0,0,0,0,0}, // born
                                   {0,0,1,1,0,0,0,0,0}  // stay
                               };

    private static readonly int[,] b2s34 =
                           {//  0 1 2 3 4 5 6 7 8
                                   {0,0,1,0,0,0,0,0,0}, // born
                                   {0,0,0,1,1,0,0,0,0}  // stay
                               };

    /// <summary>
    /// Muodostaa seuraavan vaiheen menemällä taulukkoa 
    /// rivi kerrallaan ylhäältä alas ja vasemmalta oikealle.
    /// Tulos tulee samaan taulukkoon jos seuraava = null.
    /// </summary>
    /// <param name="sukupolvi">Nykyinen vaihe, josta tilanne lasketaan</param>
    /// <param name="seuraava">Vaihe, johon uusi vaihe lasketaan.  Jos == null,
    /// niin vaihe lasketaan vaihe-taulukkoon.  Jos taulukoiden koot eivät
    /// ole samat, käyteään pienemmän kokoa</param>
    /// <param name="saannot">2x8 taulukko siitä, millä naapurimäärillä herää
    /// ja millä naapurimäärillä säilyy hengissä.</param>
    /// <example>
    /// <pre name="test">
    ///   int[,] vaihe = {
    ///                { 1,0,1,1 },
    ///                { 0,1,1,0 },
    ///                { 1,0,0,0 },
    ///                { 1,0,0,1 }
    ///              };
    ///    Sopulit.SeuraavaSukupolvi(vaihe);
    ///    Demo5.Sopulit.Jonoksi(vaihe," ",",") === "0 1 0 0,1 0 0 0,1 1 0 0,1 1 1 0";
    ///    Sopulit.SeuraavaSukupolvi(vaihe);
    ///    Demo5.Sopulit.Jonoksi(vaihe," ",",") === "0 0 0 0,1 1 0 0,0 0 1 0,0 1 1 0";
    ///    Sopulit.SeuraavaSukupolvi(vaihe);
    ///    Demo5.Sopulit.Jonoksi(vaihe," ",",") === "0 0 0 0,0 0 0 0,0 1 1 0,0 1 1 0";
    /// </pre>
    /// </example>
    public static void SeuraavaSukupolvi(int[,] sukupolvi, int[,] seuraava = null,
                      int[,] saannot = null)
    {
        if (seuraava == null) seuraava = sukupolvi;
        if (saannot == null) saannot = b3s23;
        int maxy = Math.Min(sukupolvi.GetLength(0), seuraava.GetLength(0));
        int maxx = Math.Min(sukupolvi.GetLength(1), seuraava.GetLength(1));
        for (int iy = 0; iy < maxy; iy++)
        {
            for (int ix = 0; ix < maxx; ix++)
            {
                int arvo = sukupolvi[iy, ix];
                int naapureita = LaskeNaapurit(sukupolvi, iy, ix);
                seuraava[iy, ix] = saannot[arvo, naapureita];
            }
        }
    }



    /// <summary>
    /// Laskee montako nollasta poikkeavaa naapuria on valitulla alkiolla
    /// </summary>
    /// <param name="sukupolvi">taulukko josta naapureita etsitään</param>
    /// <param name="rivi">minkä rivin naapureita etsitään</param>
    /// <param name="sarake">minkä sarakkeen naapureita etsitään</param>
    /// <returns></returns>
    /// <example>
    /// <pre name="test">
    ///   int[,] alku = {
    ///                { 1,0,1,1,0 },
    ///                { 0,1,1,0,0 },
    ///                { 1,0,0,0,0 },
    ///                { 1,0,0,0,0 }
    ///              };
    ///   Sopulit.LaskeNaapurit(alku,0,0) === 1;
    ///   Sopulit.LaskeNaapurit(alku,3,0) === 1;
    ///   Sopulit.LaskeNaapurit(alku,0,1) === 4;
    ///   Sopulit.LaskeNaapurit(alku,2,2) === 2;
    ///   Sopulit.LaskeNaapurit(alku,3,2) === 0;
    /// </pre>
    /// </example>
    public static int LaskeNaapurit(int[,] sukupolvi, int rivi, int sarake)
    {
        int naapureita = 0;
        int alkuy = rivi - 1;
        int loppuy = rivi + 1;
        int alkux = sarake - 1;
        int loppux = sarake + 1;
        if (alkuy < 0) alkuy = 0;
        if (loppuy >= sukupolvi.GetLength(0)) loppuy = sukupolvi.GetLength(0) - 1;
        if (alkux < 0) alkux = 0;
        if (loppux >= sukupolvi.GetLength(1)) loppux = sukupolvi.GetLength(1) - 1;

        for (int iy = alkuy; iy <= loppuy; iy++)
            for (int ix = alkux; ix <= loppux; ix++)
                naapureita += sukupolvi[iy, ix];
        naapureita -= sukupolvi[rivi, sarake];
        return naapureita;
    }


    /// <summary>
    /// Arvotaan taulukkoon lukuja [ala,yla] välille.
    /// </summary>
    /// <param name="taulukko">Taulukko johon lukuja arvotaan</param>
    /// <param name="ala">Pienen arvo joka taulukkoon voi tulla</param>
    /// <param name="yla">Suurin arvo joka taulukkon voi tulla</param>
    /// <example>
    /// <pre name="test">
    ///  int[,] luvut = new int[3,3];
    ///  Sopulit.Arvo(luvut,4,8);
    ///  foreach (int luku in luvut)
    ///     4 <= luku && luku <= 8 === true;
    /// </pre>
    /// </example>
    public static void Arvo(int[,] taulukko, int ala, int yla)
    {
        Random rnd = new Random();
        int ny = taulukko.GetLength(0);
        int nx = taulukko.GetLength(1);
        for (int iy = 0; iy < ny; iy++)
            for (int ix = 0; ix < nx; ix++)
                taulukko[iy, ix] = ala + rnd.Next(yla - ala + 1);
    }


    /// <summary>
    /// Täytetään taulukko valitulla arvolla
    /// </summary>
    /// <param name="taulukko">Täytettävä taulukko</param>
    /// <param name="arvo">Arvo, jolla taulukko täytetään</param>
    /// <example>
    /// <pre name="test">
    ///  int[,] luvut = new int[3,3];
    ///  Sopulit.Tayta(luvut,7);
    ///  foreach (int luku in luvut)
    ///     luku === 7;
    ///  Sopulit.Tayta(luvut,2);
    ///  foreach (int luku in luvut)
    ///     luku === 2;
    /// </pre>
    /// </example>
    public static void Tayta(int[,] taulukko, int arvo)
    {
        int ny = taulukko.GetLength(0);
        int nx = taulukko.GetLength(1);
        for (int iy = 0; iy < ny; iy++)
            for (int ix = 0; ix < nx; ix++)
                taulukko[iy, ix] = arvo;
    }
}

