using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;
using Jypeli.WP7;



/// @author  Vesa Lappalainen
/// @version 14.10.2011
/// @author Antti-Jussi Lakanen
/// @version 14.3.2012 
/// <summary>
/// Piirretään pistejoukon kuvaajaa
/// </summary>
public class G34_PNS : PhysicsGame
{
    /// <summary>Koordinaatistoakselit</summary>
    private Axis akselit;

    /// <summary>Koordinaatit piirretyille pisteille</summary>
    private List<Vector> pisteet;

    /// <summary>Merkki paikalle jota viimeksi klikattu</summary>
    private GameObject klikattuPiste;

    /// <summary>Merkki pisteelle joka on lähinnä klikattua paikkaa</summary>
    private GameObject merkkipiste;

    /// <summary>
    /// PNS-suoran f(x) = kx + b kulmakerroin k
    /// </summary>
    private double pnsK;

    /// <summary>
    /// PNS-suoran f(x) = kx + b vakio b 
    /// </summary>
    private double pnsB;

    private const double x1 = -10;
    private const double x2 = 70;
    private const double y1 = -20;
    private const double y2 = 40;
    private const double pallonKoko = (y2 - y1) / 300;

    /// <summary>
    /// Alustetaan rajat ja luodaan pisteet ja 
    /// niiden paikat pisteet-taulukkoon.
    /// </summary>
    public override void Begin()
    {
        Level.BackgroundColor = Color.White;
        SetWindowSize(800, 600);
        Level.Width = x2 - x1;
        Level.Height = y2 - y1;

        akselit = new Axis(x1, y1, x2, y2);

        pisteet = ArvoPisteet(20, x1, y1, x2, y2);

        LuoPisteet(this, pisteet, pallonKoko);

        klikattuPiste = LuoPallo(this, Vector.Zero, 1.5 * pallonKoko, Color.Blue);
        merkkipiste = LuoPallo(this, Vector.Zero, 1.5 * pallonKoko, Color.Red);

        pnsK = PNSKulmakerroin(pisteet);
        pnsB = PNSSuoranVakio(pisteet);

        Camera.ZoomTo(x1, y1, x2, y2);
        AsetaOhjaimet();
    }

    public static double PNSKulmakerroin(List<Vector> havaintopisteet)
    {
        int n = havaintopisteet.Count;
        double k;

        // k = n * sum_i=1..n x_i*y_i -

        double summaKaikkiXKertaaY = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) summaKaikkiXKertaaY += havaintopisteet[i].X * havaintopisteet[i].Y;

        double summaKaikkiX = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) summaKaikkiX += havaintopisteet[i].X;

        double summaKaikkiY = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) summaKaikkiY += havaintopisteet[i].Y;

        double xKomponenttienNeliot = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) xKomponenttienNeliot += havaintopisteet[i].X * havaintopisteet[i].X;

        k = (n * summaKaikkiXKertaaY - (summaKaikkiX * summaKaikkiY)) / (n * xKomponenttienNeliot - (summaKaikkiX * summaKaikkiX));
        return k;
    }

    public static double PNSSuoranVakio(List<Vector> havaintopisteet)
    {
        int n = havaintopisteet.Count;
        double b;

        double summaKaikkiXKertaaY = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) summaKaikkiXKertaaY += havaintopisteet[i].X * havaintopisteet[i].Y;

        double summaKaikkiX = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) summaKaikkiX += havaintopisteet[i].X;

        double summaKaikkiY = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) summaKaikkiY += havaintopisteet[i].Y;

        double xKomponenttienNeliot = 0;
        for (int i = 0; i < havaintopisteet.Count; i++) xKomponenttienNeliot += havaintopisteet[i].X * havaintopisteet[i].X;

        b = (xKomponenttienNeliot * summaKaikkiY - (summaKaikkiX * summaKaikkiXKertaaY)) / (n * xKomponenttienNeliot - (summaKaikkiX * summaKaikkiX));
        return b;
    }

    /// <summary>
    /// Asetetaan ohjaimet peliä varten. F1 ja ESC.
    /// Kuunnellaan hiiren paikkaa ja siirretään merkkipisteen sen mukaan mihin klikattu
    /// Samoin kuunnellaan puhelimelta sormella tökkäisyä.
    /// </summary>
    private void AsetaOhjaimet()
    {
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");

        Mouse.IsCursorVisible = true;
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, delegate () { SiirraMerkkipisteet(Mouse.PositionOnWorld); }, "Laita piste");

        PhoneBackButton.Listen(Exit, "Lopeta peli");
    }


    /// <summary>
    /// Siirretään sininen piste paikan p kohdalle 
    /// ja siirretään punainen piste lähimmän alueen pisteen päälle
    /// </summary>
    /// <param name="p">paikka johon sininen piste tulee</param>
    public void SiirraMerkkipisteet(Vector p)
    {
        klikattuPiste.Position = p;

        int i = LahimmanIndeksi(pisteet, p);
        if (i < 0) return;

        merkkipiste.Position = pisteet[i];
    }


    /// <summary>
    /// Luodaan peliin taulukossa olevien koordinaattien kohdalle pienet pallot
    /// </summary>
    /// <param name="game">peli, johon pisteet lisätään</param>
    /// <param name="pisteet">piirrettävien pisteiden koordinaatit</param>
    /// <param name="ala">raja jota pienemmät piirretään sinisellä</param>
    /// <param name="ala">raja jota suuremmat piirretään punaisella</param>
    /// <param name="r">säde kullekin pisteelle</param>
    public static List<GameObject> LuoPisteet(PhysicsGame game, IEnumerable<Vector> pisteet, double r, double ala = double.MinValue, double yla = double.MaxValue)
    {
        List<GameObject> pallot = new List<GameObject>();
        foreach (Vector p in pisteet)
        {
            Color color = Color.Black;
            if (p.Y <= ala) color = Color.Blue;
            if (p.Y >= yla) color = Color.Red;
            pallot.Add(LuoPallo(game, p, r, color));
        }
        return pallot;
    }


    /// <summary>
    /// Lisätään pallo ruudulle, ja palautetaan se.
    /// </summary>
    /// <param name="game">peli johon pallo lisätään</param>
    /// <param name="p">pallon keskipiste</param>
    /// <param name="r">pallon säde</param>
    /// <param name="color">pallon väri</param>
    /// <returns>lisätyn pallon viite</returns>
    public static GameObject LuoPallo(PhysicsGame game, Vector p, double r, Color color)
    {
        GameObject pallo = new GameObject(r * 2, r * 2, Shape.Circle);
        pallo.Position = p;
        pallo.Color = color;
        game.Add(pallo);
        return pallo;
    }


    /// <summary>
    /// Arvotaan joukko 2D-pisteitä halutulle alueelle
    /// </summary>
    /// <param name="n">montako pistettä arvotaan</param>
    /// <param name="x1">alueen vasemman alakulman x</param>
    /// <param name="y1">alueen vasemman alakulman y</param>
    /// <param name="x2">alueen oikean yläkulman x</param>
    /// <param name="y2">alueen oikean yläkulman y</param>
    /// <returns></returns>
    public static List<Vector> ArvoPisteet(int n, double x1, double y1, double x2, double y2)
    {
        List<Vector> pisteet = new List<Vector>(n);
        for (int i = 0; i < n; i++)
        {
            double x = RandomGen.NextDouble(x1, x2);
            double y = RandomGen.NextDouble(y1, y2);
            pisteet.Add(new Vector(x, y));
            // pisteet[i] = new Vector(x, y);
        }
        return pisteet;
    }


    /// <summary>
    /// Etsitään pistettä lähimmän taulukon alkion indeksi.
    /// </summary>
    /// <param name="pisteet">taulukko jossa pisteet, joista lähintä etsitään</param>
    /// <param name="piste">piste johon muita verrataan</param>
    /// <returns>lähimmän pisteen indeksi, -1 jos taulukko tyhjä</returns>
    /// <example>
    /// <pre name="test">
    ///   Vector[] t = { new Vector(1,2),new Vector(3,4),new Vector(5,2),new Vector(5,5) };
    ///   List<Vector> luvut = new List<Vector>(t);
    ///   PNS.LahimmanIndeksi(luvut,new Vector(3,5)) === 1;
    ///   PNS.LahimmanIndeksi(luvut,new Vector(0,0)) === 0;
    ///   PNS.LahimmanIndeksi(luvut,new Vector(5,3)) === 2;
    ///   PNS.LahimmanIndeksi(luvut,new Vector(15,5)) === 3;
    /// </pre>
    /// </example>
    public static int LahimmanIndeksi(List<Vector> pisteet, Vector piste)
    {
        int paikka = -1;
        double ero = double.MaxValue;
        for (int i = 0; i < pisteet.Count; i++)
        {
            Vector p = pisteet[i];
            double d = Vector.Distance(piste, p);
            if (d < ero)
            {
                paikka = i;
                ero = d;
            }
        }
        return paikka;
    }


    /// <summary>
    /// Tämä tehdään aina kun ruutua pitää päivittää
    /// </summary>
    /// <param name="canvas">Piirtoalue, johon voi piirtää</param>
    protected override void Paint(Canvas canvas)
    {
        akselit.Draw(canvas);
        canvas.DrawLine(-10, -10 * pnsK + pnsB, 40, 40 * pnsK + pnsB);
    }
}


/// <summary>
/// Luokka koordinaatiston piirtämistä varten
/// </summary>
public class Axis
{
    private double x1, x2, y1, y2;
    public Color Color { set; get; }

    /// <summary>
    /// Alustetaan koordinaatisto haluttuihin rajoihin
    /// </summary>
    /// <param name="x1">alueen vasemman alakulman x</param>
    /// <param name="y1">alueen vasemman alakulman y</param>
    /// <param name="x2">alueen vasemman yläkulman x</param>
    /// <param name="y2">alueen vasemman yläkulman y</param>
    public Axis(double x1, double y1, double x2, double y2)
    {
        this.x1 = x1;
        this.x2 = x2;
        this.y1 = y1;
        this.y2 = y2;
        Color = Color.Black;
    }


    /// <summary>
    /// Piirretään koordinaatisto niin, että tulee x ja y akselit ja niihin
    /// yhden välein pikkutikkuja.
    /// </summary>
    /// <param name="canvas">Canvas-olio, johon piirto tehdään</param>
    public void Draw(Canvas canvas)
    {
        canvas.BrushColor = Color;
        double t = (y2 - y1) / 200; // tikkujen koko
        double t2 = (y2 - y1) / 400; // tikkujen koko
        double tuplat = 2 * t;

        // X-akseli
        canvas.DrawLine(new Vector(x1, 0), new Vector(x2, 0));
        // Tikut X-akselille 0:sta lähtien kumpaankin suuntaan
        for (double x = 0; x >= x1; x -= 5.0) canvas.DrawLine(new Vector(x, -tuplat), new Vector(x, tuplat));
        for (double x = 0; x <= x2; x += 5.0) canvas.DrawLine(new Vector(x, -tuplat), new Vector(x, tuplat));

        for (double x = 0; x >= x1; x -= 1.0) canvas.DrawLine(new Vector(x, -t), new Vector(x, t));
        for (double x = 0; x <= x2; x += 1.0) canvas.DrawLine(new Vector(x, -t), new Vector(x, t));

        for (double x = 0; x >= x1; x -= 0.5) canvas.DrawLine(new Vector(x, -t2), new Vector(x, t2));
        for (double x = 0; x <= x2; x += 0.5) canvas.DrawLine(new Vector(x, -t2), new Vector(x, t2));

        // Y-akseli
        canvas.DrawLine(new Vector(0, y1), new Vector(0, y2));
        // Tikut Y-akselille 0:sta lähtien kumpaankin suuntaan
        for (double y = 0; y >= y1; y -= 5.0) canvas.DrawLine(new Vector(-tuplat, y), new Vector(tuplat, y));
        for (double y = 0; y <= y2; y += 5.0) canvas.DrawLine(new Vector(-tuplat, y), new Vector(tuplat, y));

        for (double y = 0; y >= y1; y -= 1.0) canvas.DrawLine(new Vector(-t, y), new Vector(t, y));
        for (double y = 0; y <= y2; y += 1.0) canvas.DrawLine(new Vector(-t, y), new Vector(t, y));

        for (double y = 0; y >= y1; y -= 0.5) canvas.DrawLine(new Vector(-t2, y), new Vector(t2, y));
        for (double y = 0; y <= y2; y += 0.5) canvas.DrawLine(new Vector(-t2, y), new Vector(t2, y));
    }
}
